#include <setjmp.h>
#include <stdio.h>
#include <stdlib.h>

/* Main program that extends example1, here the values of the 
 * buffer elements are shown.
 * The following registers are used:
 * Arithmetic registers:
 * JB_BX: ebx register
 * Address registers:
 * JB_SI: esi register
 * JB_DI: edi register
 * Special registers:
 * JB_BP: ebp, base pointer of stack
 * JB_SP: esp, stack pointer (top of stack)
 * JB_PC: pc, program counter, return address
 * 
 * For 64-Bit architectures:
 * Arithmetic registers:
 * JB_RBX: rbx register
 * General purpose register
 * JB R12
 * JB_R13
 * JB_R14
 * JB_R15
 * Special registers:
 * JB_RBP: rbp, base pointer
 * JB RSP: rsp, stack pointer (top of stack)
 * JB_PB:  pc, program counter, return address
 *
 * EAX, ECX und EDX gelten als flüchtige Register und werden
 * deshalb nicht(!) gespeichert.
 */
int main()
{
    //buffer for setjmp()/longjmp()
    //stores registers
    jmp_buf env;
    int i;

    //call setjmp
    //registers are stored to env, this is the point to that we go back
    i = setjmp(env);
    //setjmp() returns 0 in its first call, if we go back with longjmp()
    //the value is different
    printf("i = %d\n", i);

    //we exit after the first longjmp()
    if( i!= 0) exit(0);

    #ifdef __64BIT__
    //we print the defined values here
    printf("\nRBX: %8x\n", env[0].__jmpbuf[JB_RBX]);
    printf("R12: %8x\n", env[0].__jmpbuf[JB_R12]);
    printf("R13: %8x\n", env[0].__jmpbuf[JB_R13]);
    printf("R14: %8x\n", env[0].__jmpbuf[JB_R14]);
    printf("R15: %8x\n", env[0].__jmpbuf[JB_R15]);
    printf("RBP: %8x\n", env[0].__jmpbuf[JB_RBP]);
    printf("RSP: %8x\n", env[0].__jmpbuf[JB_RSP]);
    printf("PC: %8x\n\n", env[0].__jmpbuf[JB_PC]);
    #else
    //we print the defined values here
    printf("\nBX: %8x\n", env[0].__jmpbuf[JB_BX]);
    printf("SI: %8x\n", env[0].__jmpbuf[JB_SI]);
    printf("DI: %8x\n", env[0].__jmpbuf[JB_DI]);
    printf("BP: %8x\n", env[0].__jmpbuf[JB_BP]);
    printf("SP: %8x\n", env[0].__jmpbuf[JB_SP]);
    printf("PC: %8x\n\n", env[0].__jmpbuf[JB_PC]);
    #endif

    //we jump to the point that we saved with the first setjmp call
    //the return value is 2 (must be different from 0)
    longjmp(env, 2);
    
    //we cannot see it as we exit our program before
    printf("Can you see this line=\n");
}

